% Correction application Chapitre 8
% Macroéconomie quantitative (2014, éd. Ellipses) 
% par Christophe Cahn

% Ce fichier permet de reproduire les élements contenu
% dans le fichier App_8_estimation.pdf disponible sur le site
% www.christophecahn.fr/macroquant

% Filename : App_8_estimation.m
% Application : Matlab/Octave
% Date : 24-04-2014
% Christophe Cahn (c) 2014


addpath('../MMMtbx');

% conversion du fichier data.txt dans un format exploitable (.m,.dat,etc...)
data = load('../data/data.txt');
y    = data(:,1) ;
p    = data(:,2) ;
i    = data(:,3) ;
save('data','y','p','i') ;   

dynare App_8_1b     

dynare App_8_1c

dynare App_8_1d

% nettoyage
delete *.log *dynamic.m *results.mat *static.m *8_1b.m *8_1c.m *8_1d.m *variables.m *.mat *.dat
rmdir('App_8_1b','s')
rmdir('App_8_1c','s')
rmdir('App_8_1d','s')

% question 2c

% simulation d'une chronique de chocs pour en, ei et ep
% sur la longueur de l'échantillon (500) + un burning sample (200) * nbs de simulations
data = load('../data/data.txt');
y    = data(:,1) ;
p    = data(:,2) ;
i    = data(:,3) ;
save('data','y','p','i') ;   
load data ;
T    = length(y);
burn = 200 ;
nsim = 3 ;
randn('state',128); % pour la reproductibilité de l'exercice
ee = randn(3,nsim*(burn+T));
% calcul des moments sur les observations  
moments = mom([y p i]);
% rappel : ordre des coefficients
% 1:k,2:ay,3:ap,4:rp,5:rn,6:ri,7:sp,8:sn,9:si
xinit = trans([.01 .1 1.2 .8 .8 .8 .01 .01 .01]);        
xtrue = [.023 .09 1.3 .7 .8 .9 .013 .010 .007];            

opt = optimset('Display','iter',...
             'MaxFunEvals',6000,...
             'MaxIter',6000,...
             'TolX',10e-8,...
             'TolFun',10e-8);
xsol = fminsearch(@(xx) funobj(invtrans(xx),ee,moments,T,burn,nsim),xinit,opt)

% en utilisant le solver de Sims
% [ff,xsolcsminwel]=csminwel(@(xx) funobj(invtrans(xx),ee,moments,T,burn,nsim),xinit,eye(length(xinit)),[],1e-14,500);

% question 3.a

% Estimation d'un VAR(4) à partir des données 
data = load('../data/data.txt');   
[B,S,covB,covS] = estimVAR(data,4) ;
nB = length(vec(B));
nS = length(vech(S));
U  = [covB, zeros(nB,nS); zeros(nS,nB), covS];  
W  = U\eye(nB+nS);
VAR_coef_obs = [vec(B);vech(S)];    

T    = 500 ;
burn = 200 ;
nsim = 3  ;       % pour un check, choisir nsim = 1 et seed = 33
seed = 123 ;
randn('state',seed); % pour la reproductibilité de l'exercice
ee = randn(3,nsim*(burn+T));

xinit = trans([.01 .1 1.2 .8 .8 .8 .01 .01 .01]);        
xtrue = [.023 .09 1.3 .7 .8 .9 .013 .010 .007];            

opt = optimset('Display','iter',...
             'MaxFunEvals',6000,...
             'MaxIter',6000,...
             'TolX',10e-8,...
             'TolFun',10e-8);

xsolVAR = fminsearch(@(xx) funobjVAR(invtrans(xx),ee,VAR_coef_obs,4,T,burn,nsim,W),xinit,opt)

% en utilisant le solver de Sims  
% [ff,xsolVARcsminwel]=csminwel(@(xx) funobjVAR(invtrans(xx),ee,VAR_coef_obs,4,T,burn,nsim,W),xinit,eye(length(xinit)),[],1e-14,500);